%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reads data and stores it in the right format for CNN
% Before reading the data, set up the required parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% resize all images to R rows and C columns
R = 200;
C = 200;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% How many samples to use for training, testing, etc.
N = 100;              % read 100 samples per class
n = 80;               % number of training samples
m = 20;               % numberof test samples
validPercent = 30;    % percent of training samples to use for validation


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now we are ready to read the data

d = dir(dataDir);
numSubDir = size(d,1);

totalSamples = N*(numSubDir-2);
numClasses = numSubDir-2;

Y_Train = zeros(numClasses*n,1); % will store true class of each sample
Y_Test = zeros( numClasses*m,1); % will store true class of each sample
X_Train = zeros(R,C,3,numClasses*n);
X_Test = zeros(R,C,3,numClasses*m);
Y_Train_V = ones( numClasses*n,1); % 2 for samples to use fo valiation, 1 for training

nextTrainNum = 1;
nextTestNum = 1;

for i = 3:numSubDir,  % go through subdirectories
    outName = d(i).name;
    fullDirName = strcat(dataDir,'\',outName); % name of the next subdirectory
    subDir = dir(fullDirName);
    numFilesSubDir = size(subDir,1);

    if N+2 > numFilesSubDir,
        error('Not enough files in a subdirectory');
    end;
    
    locCounter = 1;
    for j = 3:N+2,  % go through all subfiles
        outName = subDir(j).name;
        fullFileName = strcat(fullDirName,'\',outName);
        im = imread(fullFileName);
        
        if size(im,3) ~= 3,   % not a color image
           im = cat(3,im,im,im);
        end;
   
        
        im = single(imresize(im,[R,C]));
    
        if locCounter <= 80, % reading training data
            X_Train(:,:,:,nextTrainNum) = im;
            Y_Train(nextTrainNum) = i-2;   % current class
            if randi(100,1) < validPercent, % use for validation
                 Y_Train_V(nextTrainNum) = 2;
             end;
            nextTrainNum = nextTrainNum+1;
        else  % started reading test data
            X_Test(:,:,:,nextTestNum) = im;
            Y_Test(nextTestNum) = i-2;   % current class
            nextTestNum = nextTestNum+1;
        end;
        
        locCounter = locCounter + 1;
    end;
end;

X_Train = single(X_Train);
imdb.images.data = X_Train;
imdb.images.labels = Y_Train';
imdb.images.set = Y_Train_V';     % sets validation samples
X_Test = single(X_Test);          % test data

